﻿using System.Reflection;

using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

using Platform.Extensions;
using Platform.Modules.Abstract;

namespace Platform.Modules.Implementation
{
    /// <summary>
    /// A plugin definition
    /// </summary>
    public abstract class ModuleBase : IModule
    {
        #region Construction & Fields

        private IModuleHost _Host;
        protected ILogger _Logger;

        /// <summary>
        /// Construct plugin
        /// </summary>
        public ModuleBase()
        {
            Folder = Assembly.GetExecutingAssembly().GetFolder();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the plugin name
        /// </summary>
        public abstract string Name { get; }

        /// <summary>
        /// Gets the plugin folder
        /// </summary>
        public string Folder { get; }

        /// <summary>
        /// Module Configuration Section
        /// </summary>
        public IConfigurationSection Configuration { get; private set; }

        #endregion

        #region Initialization

        /// <summary>
        /// First step in plugin initialization
        /// </summary>
        /// <param name="collection"></param>
        public virtual void Initialize( IModuleHost host, IServiceCollection servicesBuilder)
        {
            // store host reference
            _Host = host;

            // get our section
            Configuration = host.Configuration.GetSection(Name);

            // add plugin/host services
            servicesBuilder.AddSingleton<IModule>(this);
        }

        /// <summary>
        /// Start Module
        /// </summary>
        public virtual void Start()
        {
        }

        /// <summary>
        /// Stop Module
        /// </summary>
        public virtual void Stop()
        {
        }

        #endregion
    }
}
